package eu.ztsh.garmin

import android.annotation.SuppressLint
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothSocket
import android.util.Log
import com.mapbox.navigation.ui.maneuver.model.Maneuver
import eu.ztsh.garmin.data.Direction
import eu.ztsh.garmin.data.GarminMapper
import eu.ztsh.garmin.data.ManeuverMapper
import eu.ztsh.garmin.data.OutAngle
import eu.ztsh.garmin.data.OutType
import eu.ztsh.garmin.data.State
import java.io.IOException
import java.util.*
import java.util.concurrent.SynchronousQueue


@SuppressLint("MissingPermission")
class Garmin(
    val context: MainActivity,
    val device: BluetoothDevice,
    val adapter: BluetoothAdapter
) {

    private lateinit var connection: ConnectThread
    private lateinit var processing: ProcessingThread
    private var stateCache: State = State()

    fun start() {
        connection = ConnectThread()
        connection.start()
    }

    fun close() {
        connection.close()
    }

    fun process(maneuver: Maneuver) {
        processing = ProcessingThread(maneuver)
        processing.start()
        processing.join()
    }



    private inner class ProcessingThread(val maneuver: Maneuver) : Thread() {
        override fun run() {
            // TODO: check for equality before mapping!
            send(ManeuverMapper.apply(maneuver))
        }

        fun send(incoming: eu.ztsh.garmin.data.State) {
            if (stateCache.distance != incoming.distance) {
                setDistance(incoming)
            }
            if (stateCache.direction != incoming.direction) {
                setDirection(stateCache)
            }
            stateCache = incoming
        }


        private fun setLines(state: eu.ztsh.garmin.data.State) {

        }

        private fun setDistance(state: eu.ztsh.garmin.data.State) {
            connection.enqueue(GarminMapper.setDistance(state))
        }

        private fun setDirection(direction: eu.ztsh.garmin.data.State) {
            connection.enqueue(GarminMapper.setDirection(direction))
        }

    }

    private inner class ConnectThread : Thread() {

        private val queue: SynchronousQueue<IntArray> = SynchronousQueue()
        private var current: IntArray = intArrayOf()

        private val socket: BluetoothSocket? by lazy(LazyThreadSafetyMode.NONE) {
            context.checkBt()
            device.createRfcommSocketToServiceRecord(UUID.fromString("00001101-0000-1000-8000-00805F9B34FB"))
        }

        override fun run() {
            // Cancel discovery because it otherwise slows down the connection.
            context.checkBt()
            adapter.cancelDiscovery()
            socket?.connect()
            sleep(3000)
            readAll()
            while (true) {
                val newCurrent = Optional.ofNullable(queue.poll()).orElse(current)
                current = newCurrent
                send(current)
                sleep(900)
            }
        }

        // Closes the client socket and causes the thread to finish.
        fun close() {
            try {
                socket?.close()
            } catch (e: IOException) {
                Log.e(TAG, "Could not close the client socket", e)
            }
        }

        fun enqueue(data: IntArray) {
            queue.put(data)
        }

        private fun send(data: IntArray) {
            sendRaw(prepareData(data))
        }

        private fun readAll() {
            val buffer = ByteArray(64)
            val istr = socket!!.inputStream
            istr!!.let {
                if (it.available() > 0) {
                    it.read(buffer)
                }
            }
        }

        private fun sendRaw(buff: IntArray) {
            buff.forEach { socket!!.outputStream.write(it) }
            socket!!.outputStream.flush()
            sleep(2000)
            readAll()
        }

    }

    companion object {
        fun prepareData(input: IntArray): IntArray {
            val n = input.size
            var crc = (0xeb + n + n).toUInt()
            val chars = ArrayList<Int>()
            chars.add(0x10)
            chars.add(0x7b)
            chars.add((n + 0x06))
            if (n == 0xa)
                chars.add(0x10)
            chars.add(n)
            chars.add(0x00)
            chars.add(0x00)
            chars.add(0x00)
            chars.add(0x55)
            chars.add(0x15)
            for (char in input) {
                crc = (crc + char.toUInt())
                chars.add(char)
                if (char == 0x10)
                    chars.add(0x10)
            }
            chars.add((-(crc.toInt()) and 0xff))
            chars.add(0x10)
            chars.add(0x03)
            return chars.toIntArray()
        }

        private const val TAG = "GARMIN"
    }
}
